load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/shea_util.ncl"

lat0=stringtodouble(getenv("LAT0"))

if(tochar(getenv("figs_type")).eq."")then
   figs_type="png"
else
   figs_type=getenv("figs_type")
end if
;figs_type="png"
print(""+figs_type)

DIR_FIG=getenv("DIR_FIG")
wks   = gsn_open_wks(figs_type,DIR_FIG+"MFWT_vs_AMOCr_"+lat0+"_models_plot")
gsn_define_colormap(wks,"QTS")

;----------------------------------------------------------------------
; Set some plot resources
;----------------------------------------------------------------------

;;; 2  18+ ---> plus sign
;;; 3  1*  ---> asterisk 
;;; 5  18U ---> cross (x)
;;; 6  19Z ---> hollow square
;;; 7  19[ ---> up pointing triangle
;;; 8  19X ---> down pointing triangle
;;; 9  19\ ---> diamond
;;; 10 19` ---> left pointing filled triangle
;;; 11 19b ---> right pointing filled triangle
;;; 12 19] ---> five-pointed star
;;; 14 18Z ---> circle with center dot

;;; 17 19% ---> a heart-shaped
;;; 18 19$ ---> a spade
 
;;; 19 35p ---> a filled Plum Flower 
;;; 20 35q ---> a filled diamond
;;; 21 35r ---> a filled heart-shaped
;;; 22 35s ---> a filled spade
;;; 23 35y ---> a filled square
;;; 24 35z ---> a filled five-pointed star 

;;; 25 34u ---> a filled up pointing triangle
;;; 26 34u ---> a filled down pointing triangle

;;; 35y ---> a filled square
  mstrings = (/"%","$","p","q","r","s","y","z","u","u"/)
  fontnums = (/ 19, 19, 35, 35, 35, 35, 35, 35, 34, 34/)
  xoffset  = 0.0
  yoffset  = 0.0
  ratio    = 1.0
  sizes    = (/0.8,0.8,0.8,1.0,0.8,0.8,0.8,1.0,1.2,1.2/)
  angle    = (/0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,180/)

  new_indexes = NhlNewMarker(wks, mstrings, fontnums, xoffset, yoffset, ratio, sizes, angle)

;marker=(/2,3,5,6,23,7,25,8,26,9,20,10,11,12,24,14,17,21,18/)
;----------------------------------------------------------------------
; Set some plot resources
;----------------------------------------------------------------------
res                    = True
;res@gsnDraw            = False       ; Don't draw plot
;res@gsnFrame           = False       ; Don't advance frame
res@vpXF              = 0.18
;res@vpYF              = 0.95
;res@vpWidthF          = 0.6 
;res@vpHeightF         = 0.6 

;---Resources for X/Y axes.
;res@trXMinF            =  10.0
;res@trXMaxF            =  32.0
;res@trYMinF            = -0.7 
;res@trYMaxF            =  0.0 
res@tiXAxisString        = "Volume Transport (Sv)"
res@tiYAxisString        = "Freshwater Transport (Sv)"

;res@gsnLeftString              = ""
;res@gsnRightString             = ""
;res@gsnCenterString            = ""

res@tmYROn               = False

;res@gsFontThicknessF        =3
;res@lbLabelFontThicknessF=1
res@lgLabelFontThicknessF=1
;res@lgLineLabelFontThicknessF=1
res@tiXAxisFontThicknessF=1
res@tiYAxisFontThicknessF=1
res@tmXBLabelFontThicknessF=1
res@tmYLLabelFontThicknessF=1
res@txFontThicknessF=1
res@xyLineLabelFontThicknessF=1
;res@cnConstFLabelFontThicknessF=1
;res@cnLineLabelFontThicknessF=1

res@tmYLAutoPrecision = False
res@tmYLPrecision     = 1    

res1=res
res1@gsnDraw            = False       ; Don't draw plot
res1@gsnFrame           = False       ; Don't advance frame

res0=res
;  res@gsnCenterString   = "transport weighted T/S @ "+flt2string(lat0)+"N"
res0@xyMarkLineMode     = "Markers"
res0@xyMarkerSizeF      = 0.02
res0@xyMarkerThicknessF = 1.
res0@xyMonoMarkerColor  = False
;res0@xyMarkers          = marker
;res0@xyMarkerColors     = "white"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
str=systemfunc("echo ${MODELS[*]}")
models=str_split(str, " ")   
str=systemfunc("echo ${MODELS_RUNS[*]}")
models_runs=str_split(str, " ")   
str=systemfunc("echo ${LEGENDS[*]}")
legends=str_split(str, " ")   
str=systemfunc("echo ${TEXTS[*]}")
texts=str_split(str, " ")

;print("models:"+models)
;print("models_runs:"+models_runs)
;print("legends:"+legends)
;printVarSummary(legends)

;======================================================================
; qq(k,i,m,g)
; k ---> lev:note = "0:4, (,Smax], (Smax,S000], (S000,), (Smax,Smin], (Smin,)"
; i ---> lon:note = "0:3, net, pos, neg, abs"
; m ---> model
;======================================================================
;***************************************************************************************************************************
;    data(n1,n2,n3): n1--number of lines in each plot(i), n2---the length of the line is, n3---number of plot(i)
;***************************************************************************************************************************

num_model=dimsizes(models)

qq=new((/num_model/),"double",1e20)
MFWT=new((/6,3,num_model/),"double",1e20)
moc=new((/num_model/),"double",1e20)

if(num_model .le. 20)then
   colors=(/20,4,12,18,8,16,21,6,10,14,19,3,11,17,7,15,2,5,9,13/)
   marker=colors
   marker=16
else if(num_model .le. 100)then
   marker=(/2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9/)
   colors=(/20,20,20,20,20,20,20,20,20,20,4,4,4,4,4,4,4,4,4,4,12,12,12,12,12,12,12,12,12,12,18,18,18,18,18,18,18,18,18,18,8,8,8,8,8,8,8,8,8,8,16,16,16,16,16,16,16,16,16,16,1,1,1,1,1,1,1,1,1,1,6,6,6,6,6,6,6,6,6,6,10,10,10,10,10,10,10,10,10,10,14,14,14,14,14,14,14,14,14,14/)
else if(num_model .le. 520)then
marker=new(520,"integer")
colors=new(520,"integer")
cs=(/20,4,12,18,8,16,21,6,10,14,19,3,11,17,7,15,2,5,9,13/)
do i=1,20
   ia=(i-1)*26
   ib=i*26-1
   marker(ia:ib)=ispan(1,26,1)
   colors(ia:ib)=cs(i-1)
end do
end if
end if
end if

plot  = new((/num_model/),graphic)
txt   = new((/num_model/),graphic)

;============================================================


;  res@tmYLLabelsOn =True
;  res@tiYAxisOn    =True
;  res@tmXBLabelsOn =True
;  res@tiXAxisOn    =True

;------------------------------------------------------------------------------model
do mm=0,num_model-1
  model=models(mm)
;  print(" "+mm+" "+model)
  DIR_in1    = getenv("DIR_DAT")+model+"/"
  ncs0=systemfunc("ls "+DIR_in1+"MFWT32_high_"+models_runs(mm)+"_????-????_"+lat0+".nc|head -n 1")
;  print(""+ncs0)
  fin = addfile(ncs0,"r")
  ;double MFWT(time, lev, lat, lon) ;
  ;lev:note = "0:5, (,Smax], (Smax,S000], (S000,), (Smax,Smin], (Smin,), (-inf,inf)
  ;lon:note = "0:2, net, pos, neg
  MFWT(:,:,mm) = dim_avg_n_Wrap(fin->MFWT(:,:,0,:),0)

  ncs1=systemfunc("ls "+DIR_in1+"AMOC32_high_"+models_runs(mm)+"_????-????_"+lat0+".nc|head -n 1")
;  print(""+ncs1)
  fin = addfile(ncs1,"r")
  moc(mm) = dim_avg_n_Wrap(fin->moc_max(:,0),0)
end do ;;; model loop

MFWT_mod=MFWT(:,:,0)
moc_mod=avg(moc)
do i=0,5
do j=0,2
   MFWT_mod(i,j)=avg(MFWT(i,j,:))
end do   
end do   

;------------------------------------------------------------------------------hycom
DIR_in0    = getenv("DIR_REF")
nc0="MFWT32_high_hycom_R1_0001-0005_"+lat0+".nc"
fin = addfile(DIR_in0+nc0,"r")
MFWT_ref = dim_avg_n_Wrap(fin->MFWT(:,:,0,:),0)

nc1="AMOC32_high_hycom_R1_0001-0005_"+lat0+".nc"
fin = addfile(DIR_in0+nc1,"r")
moc_ref = dim_avg_n_Wrap(fin->moc_max(:,0),0)

;lev:note = "0:5, (,Smax], (Smax,S000], (S000,), (Smax,Smin], (Smin,), (-inf,inf)
;lon:note = "0:2, net, pos, neg
lev0=5
lon0=0

;------------------------------------------------------------------------------Axis range
Xmin=min((/min(moc),min(moc_ref)/))
Xmax=max((/max(moc),max(moc_ref)/))
Ymin=min((/min(MFWT(lev0,lon0,:)),min(MFWT_ref(lev0,lon0))/))
Ymax=max((/max(MFWT(lev0,lon0,:)),max(MFWT_ref(lev0,lon0))/))
Xmin=floor(Xmin)
Xmax=ceil(Xmax)
Ymin=floor(Ymin*10)/10.
Ymax=ceil(Ymax*10)/10.
print("X:"+Xmin+" "+Xmax+" Y:"+Ymin+" "+Ymax)
res0@trXMinF = Xmin
res0@trXMaxF = Xmax
res0@trYMinF = Ymin
res0@trYMaxF = Ymax
;------------------------------------------------------------------------------Axis range


res0@gsnDraw            = False       ; Don't draw plot
res0@gsnFrame           = False       ; Don't advance frame
res0@xyMarkerColor = "black"
res0@xyMarker = 16
;print("moc_ref:"+moc_ref+" MFWT_ref:"+MFWT_ref(lev0,lon0))
plot0 = gsn_csm_xy (wks, (/moc_ref,moc_ref/), (/MFWT_ref(lev0,lon0),MFWT_ref(lev0,lon0)/),res0)

res0@xyMarker = 4
plot1 = gsn_csm_xy (wks, (/moc_mod,moc_mod/), (/MFWT_mod(lev0,lon0),MFWT_mod(lev0,lon0)/),res0)
overlay(plot0,plot1)


res@gsMarkerThicknessF = 1 
res@gsMarkerSizeF = 0.02
;------------------------------------------------------------------------------model
do mm=0,num_model-1
  model=models(mm)
  res@gsMarkerIndex = marker(mm)
  res@gsMarkerColor = colors(mm)
;  print(" "+mm+" "+model)
  
;  print("moc:"+moc(0:1,1,mm)+" MFWT:"+MFWT(0:1,1,mm))
  if(all(ismissing(moc(mm))) .or. all(ismissing(MFWT(lev0,lon0,mm))))then
     continue
  end if
  plot(mm) = gsn_add_polymarker(wks,plot0,moc(mm),MFWT(lev0,lon0,mm),res)
;  print(models(mm)+ " "+ moc(mm)+ " "+MFWT(lev0,lon0,mm))
end do ;;; model loop

;----------------------------------------------------------------------
; Draw some markers and text.
;----------------------------------------------------------------------
mkres = True
txres = True ; Text resources
txres@txFontHeightF = 0.012
txres@txFontThicknessF=1
txres@txJust= "CenterLeft"

colorss=new(num_model+2,typeof(colors))
markers=new(num_model+2,typeof(marker))
instits=new(num_model+2,typeof(models))
colorss(0:num_model-1)=(/colors(0:num_model-1)/)
markers(0:num_model-1)=(/marker(0:num_model-1)/)
instits(0:num_model-1)=(/legends(:)/)
colorss(num_model)=1
markers(num_model)=4
instits(num_model)="Average"
colorss(num_model+1)=1
markers(num_model+1)=16
instits(num_model+1)="high-res. model"

xpos= 0.795
ypos=fspan(0.205,0.80,num_model+2)
ypos=ypos(::-1)
;print(""+ypos)
if(num_model.gt.20)then
mkres@gsMarkerSizeF= 0.02*20/num_model
txres@txFontHeightF = 0.012*20/num_model
end if
do mm=0,num_model+1
  mkres@gsMarkerThicknessF = 1.0 
  mkres@gsMarkerIndex= markers(mm)
  mkres@gsMarkerColor= colorss(mm)
  xx0 = xpos
  yy0 = ypos(mm)
  gsn_polymarker_ndc(wks, xx0,yy0,mkres)
  gsn_text_ndc(wks,instits(mm), xx0+0.02,yy0,txres)
end do

;======================================================================
xx=ndtooned(moc)
yy=ndtooned(MFWT(lev0,lon0,:))
idx=ind(.not.ismissing(xx) .and. .not.ismissing(yy))
x=xx(idx)
y=yy(idx)
delete(xx)
delete(yy)
x0=avg(x)
y0=avg(y)
rc =  regline_stats(x,y)
rrr=(/rc/)

nx   = dimsizes(rc@Yest)
pltarry = new ( (/1,nx/), typeof(x))
pltarry(0,:) = rc@Yest

res1@xyMarkLineModes= "Lines"
res1@xyDashPatterns = 0  ; solid line
res1@xyLineThicknesses   = (/3/)
res1@xyLineColors   = (/"black"/)
res1@trXMinF   =  res@trXMinF
res1@trXMaxF   =  res@trXMaxF
plot1  = gsn_csm_xy (wks,x,pltarry,res1) ; create plot
overlay(plot0,plot1)
delete(pltarry)

txres = True ; Text resources
txres@txFontHeightF = 0.012
txres@txJust= "CenterLeft"
xpos = 0.3
if(abs(rc@tval(1)).gt.2.093)then
sig="Yes"
else
sig="No"
end if

rcf=sprintf("%5.2f",rc*100.)
tval=sprintf("%5.2f",rc@tval(1))

ypos = 0.25
;gsn_text_ndc(wks,"Reg Coef="+rcf+" "+sig, xpos,ypos,txres)
;print("Reg Coef="+rcf*100.)
yintercept=sprintf("%6.2f",rc@yintercept*100.)
print("Y="+rcf+"*X+"+yintercept)
ypos = 0.2
;gsn_text_ndc(wks,"Tval="+tval+" Tref=1.734", xpos,ypos,txres)
if(abs(rc@tval(1)).gt.2.093)then
  sig="Yes"
else
  sig="No"
end if
print(lat0+" Tval="+tval+" "+sig)
delete(ypos)
delete(rc)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      resourse for Panel plot
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;resP                            = True
;resP@gsnMaximize                = True
;resP@gsnPanelYWhiteSpacePercent = 0.001
;resP@gsnPanelXWhiteSpacePercent = 0.001
;resP@gsnPanelDebug     = True
;resP@gsnPanelXF=XFs
;resP@gsnPanelYF=YFs
;resP@gsnPanelCenter = False
;gsn_panel(wks,plot0,(/1,1/),resP)

pres=True
pres@gsnPanelDebug =True
maximize_output(wks,pres)
