load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/shea_util.ncl"

lat0=stringtodouble(getenv("LAT0"))

if(tochar(getenv("figs_type")).eq."")then
   figs_type="png"
else
   figs_type=getenv("figs_type")
end if
;figs_type="png"
print(""+figs_type)

DIR_FIG=getenv("DIR_FIG")
wks   = gsn_open_wks(figs_type,DIR_FIG+"Twgt_vs_Swgt_"+lat0+"_models_plot")
gsn_define_colormap(wks,"QTS")

;----------------------------------------------------------------------
; Set some plot resources
;----------------------------------------------------------------------

;;; 2  18+ ---> plus sign
;;; 3  1*  ---> asterisk 
;;; 5  18U ---> cross (x)
;;; 6  19Z ---> hollow square
;;; 7  19[ ---> up pointing triangle
;;; 8  19X ---> down pointing triangle
;;; 9  19\ ---> diamond
;;; 10 19` ---> left pointing filled triangle
;;; 11 19b ---> right pointing filled triangle
;;; 12 19] ---> five-pointed star
;;; 14 18Z ---> circle with center dot

;;; 17 19% ---> a heart-shaped
;;; 18 19$ ---> a spade
 
;;; 19 35p ---> a filled Plum Flower 
;;; 20 35q ---> a filled diamond
;;; 21 35r ---> a filled heart-shaped
;;; 22 35s ---> a filled spade
;;; 23 35y ---> a filled square
;;; 24 35z ---> a filled five-pointed star 

;;; 25 34u ---> a filled up pointing triangle
;;; 26 34u ---> a filled down pointing triangle

;;; 35y ---> a filled square
  mstrings = (/"%","$","p","q","r","s","y","z","u","u"/)
  fontnums = (/ 19, 19, 35, 35, 35, 35, 35, 35, 34, 34/)
  xoffset  = 0.0
  yoffset  = 0.0
  ratio    = 1.0
  sizes    = (/0.8,0.8,0.8,1.0,0.8,0.8,0.8,1.0,1.2,1.2/)
  angle    = (/0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,180/)

  new_indexes = NhlNewMarker(wks, mstrings, fontnums, xoffset, yoffset, ratio, sizes, angle)

;----------------------------------------------------------------------
; Set some plot resources
;----------------------------------------------------------------------
res                    = True
;res@gsnDraw            = False       ; Don't draw plot
;res@gsnFrame           = False       ; Don't advance frame
res@vpXF              = 0.18 
;res@vpYF              = 0.95
;res@vpWidthF          = 0.6 
;res@vpHeightF         = 0.6 

;---Resources for X/Y axes.
;res@trYMinF            =  1.8 
;res@trYMaxF            =  22. 
;res@trXMinF            =  34.8
;res@trXMaxF            =  37. 
res@tiXAxisString        = "Salinity (psu)"
res@tiYAxisString        = "Pot. Temp. (~S~o~N~C)"

;res@gsnLeftString              = ""
;res@gsnRightString             = ""
;res@gsnCenterString            = ""

res@tmYROn               = False

res@gsFontThicknessF        =3
res@lbLabelFontThicknessF=1
res@lgLabelFontThicknessF=1
res@lgLineLabelFontThicknessF=1
res@tiXAxisFontThicknessF=1
res@tiYAxisFontThicknessF=1
res@tmXBLabelFontThicknessF=1
res@tmYLLabelFontThicknessF=1
res@txFontThicknessF=1
res@xyLineLabelFontThicknessF=1
res@cnConstFLabelFontThicknessF=1
res@cnLineLabelFontThicknessF=1

res0=res
;  res@gsnCenterString   = "transport weighted T/S @ "+flt2string(lat0)+"N"
res0@xyMarkLineMode     = "Markers"
res0@xyMarkerSizeF      = 0.02
res0@xyMarkerThicknessF = 1.
res0@xyMonoMarkerColor  = False
;res0@xyMarkers          = marker
;res0@xyMarkerColors     = "white"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
str=systemfunc("echo ${MODELS[*]}")
models=str_split(str, " ")   
str=systemfunc("echo ${MODELS_RUNS[*]}")
models_runs=str_split(str, " ")   
str=systemfunc("echo ${LEGENDS[*]}")
legends=str_split(str, " ")   
str=systemfunc("echo ${TEXTS[*]}")
texts=str_split(str, " ")

;print("models:"+models)
;print("models_runs:"+models_runs)
;print("legends:"+legends)
;printVarSummary(legends)

;======================================================================
; qq(k,i,m,g)
; k ---> lev:note = "0:4, (,Smax], (Smax,S000], (S000,), (Smax,Smin], (Smin,)"
; i ---> lon:note = "0:3, net, pos, neg, abs"
; m ---> model
;======================================================================
;***************************************************************************************************************************
;    data(n1,n2,n3): n1--number of lines in each plot(i), n2---the length of the line is, n3---number of plot(i)
;***************************************************************************************************************************

num_model=dimsizes(models)

qq=new((/num_model/),"double",1e20)
tt=new((/5,4,num_model/),"double",1e20)
ss=new((/5,4,num_model/),"double",1e20)

if(num_model .le. 20)then
   colors=(/20,4,12,18,8,16,21,6,10,14,19,3,11,17,7,15,2,5,9,13/)
   marker=colors
   marker=16
else if(num_model .le. 100)then
   marker=(/2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9,2,3,5,6,23,7,25,8,26,9/)
   colors=(/20,20,20,20,20,20,20,20,20,20,4,4,4,4,4,4,4,4,4,4,12,12,12,12,12,12,12,12,12,12,18,18,18,18,18,18,18,18,18,18,8,8,8,8,8,8,8,8,8,8,16,16,16,16,16,16,16,16,16,16,1,1,1,1,1,1,1,1,1,1,6,6,6,6,6,6,6,6,6,6,10,10,10,10,10,10,10,10,10,10,14,14,14,14,14,14,14,14,14,14/)
else if(num_model .le. 520)then
marker=new(520,"integer")
colors=new(520,"integer")
cs=(/20,4,12,18,8,16,21,6,10,14,19,3,11,17,7,15,2,5,9,13/)
do i=1,20
   ia=(i-1)*26
   ib=i*26-1
   marker(ia:ib)=ispan(1,26,1)
   colors(ia:ib)=cs(i-1)
end do
end if
end if
end if

plot  = new((/num_model/),graphic)
txt   = new((/num_model/),graphic)

;============================================================


;  res@tmYLLabelsOn =True
;  res@tiYAxisOn    =True
;  res@tmXBLabelsOn =True
;  res@tiXAxisOn    =True

;------------------------------------------------------------------------------model
do mm=0,num_model-1
  model=models(mm)
;  print(" "+mm+" "+model)
  DIR_in1    = getenv("DIR_DAT")+model+"/"
  ncs0=systemfunc("ls "+DIR_in1+"thetao32_wgt_high_"+models_runs(mm)+"_????-????_"+lat0+".nc|head -n 1")
;  print(""+ncs0)
  fin = addfile(ncs0,"r")
  tt(:,:,mm) = dim_avg_n_Wrap(fin->thetao(:,:,0,:),0)

  ncs1=systemfunc("ls "+DIR_in1+"so32_wgt_high_"+models_runs(mm)+"_????-????_"+lat0+".nc|head -n 1")
;  print(""+ncs1)
  fin = addfile(ncs1,"r")
  ss(:,:,mm) = dim_avg_n_Wrap(fin->so(:,:,0,:),0)
end do ;;; model loop

tt_mod=tt(:,:,0)
ss_mod=ss(:,:,0)
do i=0,4
do j=0,3
   tt_mod(i,j)=avg(tt(i,j,:))
   ss_mod(i,j)=avg(ss(i,j,:))
end do   
end do   

;------------------------------------------------------------------------------hycom
DIR_in0    = getenv("DIR_REF")
nc0="thetao32_wgt_high_hycom_R1_0001-0005_"+lat0+".nc"
fin = addfile(DIR_in0+nc0,"r")
tt_ref = dim_avg_n_Wrap(fin->thetao(:,:,0,:),0)

nc1="so32_wgt_high_hycom_R1_0001-0005_"+lat0+".nc"
fin = addfile(DIR_in0+nc1,"r")
ss_ref = dim_avg_n_Wrap(fin->so(:,:,0,:),0)

;float thetao(time, lev, lat, lon) ;
;lev:note = "0:4, (,Smax], (Smax,S000], (S000,), (Smax,Smin], (Smin,)" ;
;lon:note = "0:3, net, pos, neg, abs" ;

lev0=0
lon0=3
lon1=1
lon2=2

;------------------------------------------------------------------------------Axis range
;Xmin=min((/min(ss(0:1,1:2,:)),min(ss_ref(0:1,1:2))/))
;Xmax=max((/max(ss(0:1,1:2,:)),max(ss_ref(0:1,1:2))/))
;Ymin=min((/min(tt(0:1,1:2,:)),min(tt_ref(0:1,1:2))/))
;Ymax=max((/max(tt(0:1,1:2,:)),max(tt_ref(0:1,1:2))/))

Xmin=min((/min(ss(0,lon1,:)),min(ss(1,lon2,:)),min(ss_ref(0,lon1)),min(ss_ref(1,lon2))/))
Xmax=max((/max(ss(0,lon1,:)),max(ss(1,lon2,:)),max(ss_ref(0,lon1)),max(ss_ref(1,lon2))/))
Ymin=min((/min(tt(0,lon1,:)),min(tt(1,lon2,:)),min(tt_ref(0,lon1)),min(tt_ref(1,lon2))/))
Ymax=max((/max(tt(0,lon1,:)),max(tt(1,lon2,:)),max(tt_ref(0,lon1)),max(tt_ref(1,lon2))/))

Xmin=floor(Xmin*100)/100.
Xmax=ceil(Xmax*100)/100.
Ymin=floor(Ymin)
Ymax=ceil(Ymax)
print("X:"+Xmin+" "+Xmax+" Y:"+Ymin+" "+Ymax)
res0@trXMinF = Xmin
res0@trXMaxF = Xmax
res0@trYMinF = Ymin
res0@trYMaxF = Ymax
;------------------------------------------------------------------------------Axis range

res0@gsnDraw            = False       ; Don't draw plot
res0@gsnFrame           = False       ; Don't advance frame
res0@xyMarkerColor = "black"
res0@xyMarker = 16
print("ss_ref:"+ss_ref(0:1,lon0)+" tt_ref:"+tt_ref(0:1,lon0))
;plot0 = gsn_csm_xy (wks, (/ss_ref(0:1,lon0)/), (/tt_ref(0:1,lon0)/),res0)
plot0 = gsn_csm_xy (wks, (/ss_ref(0,lon1),ss_ref(1,lon2)/), (/tt_ref(0,lon1),tt_ref(1,lon2)/),res0)

res0@xyMarker = 4
print("ss_mod:"+ss_mod(0:1,lon0)+" tt_mod:"+tt_mod(0:1,lon0))
;plot1 = gsn_csm_xy (wks, (/ss_mod(0:1,lon0)/), (/tt_mod(0:1,lon0)/),res0)
plot1 = gsn_csm_xy (wks, (/ss_mod(0,lon1),ss_mod(1,lon2)/), (/tt_mod(0,lon1),tt_mod(1,lon2)/),res0)

overlay(plot0,plot1)

res@gsMarkerThicknessF = 1  
res@gsMarkerSizeF = 0.02
;------------------------------------------------------------------------------model
do mm=0,num_model-1
  model=models(mm)
  res@gsMarkerIndex = marker(mm)
  res@gsMarkerColor = colors(mm)
;  print(" "+mm+" "+model)
  
;  print("ss:"+ss(0:1,1,mm)+" tt:"+tt(0:1,1,mm))
  if(all(ismissing(ss(0:1,1,mm))) .or. all(ismissing(tt(0:1,1,mm))))then
     continue
  end if
;  plot(mm) = gsn_add_polymarker(wks,plot0,ss(0:1,1,mm),tt(0:1,1,mm),res)
  plot(mm) = gsn_add_polymarker(wks,plot0,(/ss(0,lon1,mm),ss(1,lon2,mm)/),(/tt(0,lon1,mm),tt(1,lon2,mm)/),res)
  print(models(mm)+ " "+ ss(1,1,mm)+ " "+tt(1,1,mm))
end do ;;; model loop

;----------------------------------------------------------------------
; Draw some markers and text.
;----------------------------------------------------------------------
mkres = True
txres = True ; Text resources
txres@txFontHeightF = 0.012
txres@txFontThicknessF=1
txres@txJust= "CenterLeft"

colorss=new(num_model+2,typeof(colors))
markers=new(num_model+2,typeof(marker))
instits=new(num_model+2,typeof(models))
colorss(0:num_model-1)=(/colors(0:num_model-1)/)
markers(0:num_model-1)=(/marker(0:num_model-1)/)
instits(0:num_model-1)=(/legends(0:num_model-1)/)
colorss(num_model)=1
markers(num_model)=4
instits(num_model)="Average"
colorss(num_model+1)=1
markers(num_model+1)=16
instits(num_model+1)="high-res. model"

xpos= 0.795
ypos=fspan(0.155,0.80,num_model+2)
ypos=ypos(::-1)
;print(""+ypos)
if(num_model.gt.20)then
mkres@gsMarkerSizeF= 0.02*20/num_model
txres@txFontHeightF = 0.012*20/num_model
end if
do mm=0,num_model+1
  mkres@gsMarkerThicknessF = 1.0 
  mkres@gsMarkerIndex= markers(mm)
  mkres@gsMarkerColor= colorss(mm)
  xx0 = xpos
  yy0 = ypos(mm)
  gsn_polymarker_ndc(wks, xx0,yy0,mkres)
  gsn_text_ndc(wks,instits(mm), xx0+0.02,yy0,txres)
end do

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      resourse for Panel plot
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;resP                            = True
;resP@gsnMaximize                = True
;resP@gsnPanelYWhiteSpacePercent = 0.001
;resP@gsnPanelXWhiteSpacePercent = 0.01
;resP@gsnPanelDebug     = True
;resP@gsnPanelXF=XFs
;resP@gsnPanelYF=YFs
;resP@gsnPanelCenter = False
;gsn_panel(wks,plot0,(/1,1/),resP)

pres=True
pres@gsnPanelDebug =True
;pres@gsnPanelXWhiteSpacePercent = 0.01
maximize_output(wks,pres)
