#!/bin/bash

source /${driver}

export DIR_REF="${DIR_REFS}/"
echo ${DIR_REF}

ncls=(thetao_lat0_climate_xz_plot.ncl
so_lat0_climate_xz_plot.ncl
vmo_lat0_climate_xz_plot.ncl
sig2_lat0_climate_xz_plot.ncl
Q_r_high_lat0_models_plot.ncl
S_r_high_lat0_models_plot.ncl
T_r_high_lat0_models_plot.ncl
trans_lat0_climate_TS_plot.ncl
AMOC_r_high_lat0_models_plot.ncl
MHT_vs_AMOCr_lat0_models_plot.ncl
MHT_vs_Tdiff_lat0_models_plot.ncl
MFWT_vs_AMOCr_lat0_models_plot.ncl
MFWT_vs_Sdiff_lat0_models_plot.ncl
Twgt_vs_Swgt_lat0_models_plot.ncl)

#ncls=()

num_model=${#models[@]}
num_run=${#runs[@]}
echo "number of models: ${num_model}"
echo "number of   runs: ${num_run}"
((num_model--))

#--- use model alias instead of model's name for plotting, if models_alias defined
export legend_models=${models[*]}
#[[ ! -z ${models_alias} ]] && legend_models=${models_alias[@]:0:${num_model}}
[[ ! -z ${models_alias} ]] && legend_models=(${models_alias[@]})
declare -a run_alias
[[ ! -z ${runs_alias} ]] && run_alias=(${runs_alias[@]})
echo "legend_models: ${legend_models[*]}"
echo "run_alias: ${run_alias[*]}"

resol=`echo ${resols[*]} | awk '{print $NF}'`

for exp in ${exps[*]};do

export DIR_DAT=${DIR_DIAG}/${exp}/

[[ ! -d ${DIR_FIGS}/${exp}/images ]] && \cp -R ${DIR_HTML}/images ${DIR_FIGS}/${exp}/

#####################################################################################################
case ${compare} in

#==================================================================================================== 1
#--- RUN COMPARISION, usually for internal variability. each model with all its runs into one figures, resulting in many figures.
1)
\cp -f ${DIR_HTML}/compare1/compare1_?.htm ${DIR_FIGS}/${exp}/
:>${DIR_FIGS}/${exp}/compare1_2_models.htm

cnt=1
for model in ${models[*]};do
   export DIR_FIG=${DIR_FIGS}/${exp}/compare1/${model}/
   [[ ! -d ${DIR_FIG} ]] && mkdir -p ${DIR_FIG}
   echo "DIR_FIG= ${DIR_FIG}"

   \cp -f ${DIR_FIGS}/${exp}/compare1_2.htm ${DIR_FIGS}/${exp}/compare1_2_tmp.htm
   sed -i "s#>1#>${cnt}#g" ${DIR_FIGS}/${exp}/compare1_2_tmp.htm
   sed -i "s#CCSM4#${model}#g" ${DIR_FIGS}/${exp}/compare1_2_tmp.htm
   cat ${DIR_FIGS}/${exp}/compare1_2_tmp.htm >> ${DIR_FIGS}/${exp}/compare1_2_models.htm
   rm -f ${DIR_FIGS}/${exp}/compare1_2_tmp.htm
   ((cnt++))

   \cp -f ${DIR_HTML}/compare1/sets_MODEL.htm ${DIR_FIGS}/${exp}/compare1/sets_${model}.htm
   sed -i "s#CCSM4#${model}#g" ${DIR_FIGS}/${exp}/compare1/sets_${model}.htm

   \cp -f ${DIR_HTML}/compare1/set?.htm ${DIR_FIG}
   sed -i "s#CCSM4#${model}#g" ${DIR_FIG}/set?.htm

   export model=${model}

   legend_model=${model}

   #---                                   MFWT64_high_ACCESS1-0_r1i1p1_1850-2005_26.5.nc
   num=`ls ${DIR_DIAG}/${exp}/${model}/MFWT64_${resol}_${model}_*_${LAT0}.nc|wc -l`
   [[ ${num} -eq 0 ]] && echo "${model} has no runs." && break
   #--- if runs_alias defined, take its first $num values, else, use run names in nc files.

    n=0
    unset mruns
    for nc in `ls ${DIR_DIAG}/${exp}/${model}/MFWT64_${resol}_${model}_*_${LAT0}.nc`;do
        nc=${nc##*_${model}_}
        nc=${nc%%_*}
        mruns[${n}]=${nc}
        ((n++))
    done
    echo "mruns=${mruns[@]}"

   if [[ ! -z ${run_alias} ]];then
      echo "1"
      legend_runs=(${runs_alias[@]:0:${num}})
   else
      echo "0"
      legend_runs=${mruns[*]}
   fi
   echo "legend_runs= ${legend_runs[@]}"

   m=0
   ((num--))
   unset models_runs legends modelss
   for n in `seq 0 ${num}`;do
      echo $n
      models_runs[${m}]="${model}_${mruns[${n}]}"
      legends[${m}]="${legend_runs[${n}]}"
      modelss[${m}]="${model}"
      ((m++))
   done

   export MODELS_RUNS=${models_runs[*]}
   export LEGENDS=${legends[*]}
   export MODELS=${modelss[*]}
   export TEXTS=${legend_runs[*]}
   echo "MODELS_RUNS= ${MODELS_RUNS[*]}"
   echo "LEGENDS= ${LEGENDS[*]}"
   echo "MODELS= ${MODELS[*]}"

   for ncl in ${ncls[*]};do
      echo "--------------------------------------------------running ${model} ${ncl}"
      pic=${ncl/lat0/${LAT0}}
      echo "rm -f *${pic%.ncl}*"
      \rm -f `ls ${DIR_FIG}*${pic%.ncl}.pdf`
      \rm -f `ls ${DIR_FIG}*${pic%.ncl}.gif`
      ls "${DIR_FIG}*${pic%.ncl}*"
      time ncl ${ncl}
      echo "--------------------------------------------------DONE ${model} ${ncl}"
   done

   cd $DIR_FIG

   rm -f crop_crop_*.pdf
   for pdf in `ls *pdf|grep -v crop`;do
       [[ ! -e crop_${pdf//crop_/} ]] && pdfcrop --margins 10 ${pdf} crop_${pdf} && echo "pdfcrop ${pdf} to ${pdf%.pdf}.gif"
       [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY -limit memory 64 -limit map 128 crop_${pdf} ${pdf%.pdf}.gif && echo "converting crop_${pdf} to ${pdf%.pdf}.gif"
#       [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY crop_${pdf} ${pdf%.pdf}.gif
#       convert -density $DENSITY -resize 220x170 crop_${pdf} thumbnail_${pdf%.pdf}.gif
   done    

   cd -

   ((c++))

done     

cat ${DIR_FIGS}/${exp}/compare1_1.htm ${DIR_FIGS}/${exp}/compare1_2_models.htm ${DIR_FIGS}/${exp}/compare1_3.htm > ${DIR_FIGS}/${exp}/compare1.htm
rm -f ${DIR_FIGS}/${exp}/compare1_*.htm
echo "===================================================================================================="
echo "DONE: compare1, MODEL COMPARISION, only the first run of each model are used for plotting."
echo "===================================================================================================="

;;

#==================================================================================================== 2
2) 
echo "MODEL COMPARISION, only the first run of each model are used for plotting."
#--- since only one run plotted for each model, then model name itself is enough to be a distinction
#--- thus, final legend = model_names
for n in `seq 0 ${num_model}`;do
   models_runs[${n}]="${models[$n]}_${runs[0]}"
   legends[${n}]="${legend_models[$n]}"
done

export MODELS_RUNS=${models_runs[*]}
export MODELS=${models[*]}
export LEGENDS=${legends[*]}
export TEXTS=${models[*]}
export DIR_FIG=${DIR_FIGS}/${exp}/compare2/
[[ ! -d ${DIR_FIG} ]] && mkdir -p ${DIR_FIG}
echo ${MODELS_RUNS}
echo ${LEGENDS}
echo ${DIR_FIG}
echo ${DIR_HTML}

\cp -n ${DIR_HTML}/compare2/compare2.htm ${DIR_FIGS}/${exp}/
\cp -n ${DIR_HTML}/compare2/set?.htm ${DIR_FIG}

#rm -f *.pdf
for ncl in ${ncls[*]};do
   echo "--------------------------------------------------running ${ncl}"
   pic=${ncl/lat0/${LAT0}}
   echo "rm -f *${pic%.ncl}*"
   \rm -f `ls ${DIR_FIG}*${pic%.ncl}.pdf`
   \rm -f `ls ${DIR_FIG}*${pic%.ncl}.gif`
   ls "${DIR_FIG}*${pic%.ncl}*"
   time ncl ${ncl}
   echo "--------------------------------------------------DONE ${ncl}"
done

cd $DIR_FIG

rm -f crop_crop_*
for pdf in `ls *pdf|grep -v crop`;do
    [[ ! -e crop_${pdf//crop_/} ]] && pdfcrop --margins 10 ${pdf} crop_${pdf} && echo "pdfcrop ${pdf} to crop_${pdf}"
    [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY -limit memory 64 -limit map 128 crop_${pdf} ${pdf%.pdf}.gif && echo "converting crop_${pdf} to ${pdf%.pdf}.gif"
#    convert -density $DENSITY crop_${pdf} ${pdf%.pdf}.gif
#    [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY crop_${pdf} ${pdf%.pdf}.gif 
#    [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY -dMaxBitmap=5000000000 crop_${pdf} ${pdf%.pdf}.gif 
#    convert -density $DENSITY -resize 220x170 crop_${pdf} thumbnail_${pdf%.pdf}.gif ### NOT BEST
done    

echo "===================================================================================================="
echo "DONE: compare2, MODEL COMPARISION, only the first run of each model are used for plotting."
echo "===================================================================================================="

cd -
;;

#==================================================================================================== 3
#---  all models and all runs into one figure. DO NOT RECOMMAND, if too many models and runs.  
3)
m=0
c=0
export DIR_FIG=${DIR_FIGS}/${exp}/compare3/
[[ ! -d ${DIR_FIG} ]] && mkdir -p ${DIR_FIG}
echo ${DIR_FIG}

\cp -n ${DIR_HTML}/compare3/compare3.htm ${DIR_FIGS}/${exp}/
\cp -n ${DIR_HTML}/compare3/set?.htm ${DIR_FIG}

for model in ${models[*]};do
   echo "${model}"
   [[ ! -z ${models_alias} ]] && legend_model=${legend_models[${c}]} || legend_model=${model}
   echo "legend_model= ${legend_model}"
   #---                                   MFWT64_high_ACCESS1-0_r1i1p1_1850-2005_26.5.nc
   num=`ls ${DIR_DIAG}/${exp}/${model}/MFWT64_${resol}_${model}_*_${LAT0}.nc|wc -l`
   echo "num= ${num}"
   [[ ${num} -eq 0 ]] && echo "${model} has no runs." && break
   #--- if runs_alias defined, take its first $num values, else, use run names in nc files.
    n=0
    for nc in `ls ${DIR_DIAG}/${exp}/${model}/MFWT64_${resol}_${model}_*_${LAT0}.nc`;do
        nc=${nc##*_${model}_}
        nc=${nc%%_*}
        mruns[${n}]=${nc}
        ((n++))
    done
    echo "mruns= ${mruns[@]}"

   if [[ ! -z ${run_alias[@]} ]];then
      echo "1"
      legend_runs=(${run_alias[@]:0:${num}})
   else
      echo "0"
      legend_runs=${mruns[*]}
   fi
   echo "legend_runs= ${legend_runs[@]}"

   ((num--))
   for n in `seq 0 ${num}`;do
      models_runs[${m}]="${model}_${mruns[${n}]}"
      legends[${m}]="${legend_model}_${legend_runs[${n}]}"
      modelss[${m}]="${model}"
      ((m++))
   done

   echo "models_runs= ${models_runs[@]}"
   echo "legends= ${legends[@]}"
   echo "modelss= ${modelss[@]}"

   ((c++))

done

export MODELS_RUNS=${models_runs[*]}
export LEGENDS=${legends[*]}
export MODELS=${modelss[*]}
export TEXTS=${models_runs[*]}
echo "MODELS_RUNS: ${MODELS_RUNS[*]}"
echo "LEGENDS: ${LEGENDS[*]}"
echo "MODELS: ${MODELS[*]}"
echo "TEXTS: ${TEXTS[*]}"

for ncl in ${ncls[*]};do
    echo "--------------------------------------------------running ${ncl}"
   pic=${ncl/lat0/${LAT0}}
   echo "rm -f *${pic%.ncl}*"
   \rm -f `ls ${DIR_FIG}*${pic%.ncl}.pdf`
   \rm -f `ls ${DIR_FIG}*${pic%.ncl}.gif`
   ls "${DIR_FIG}*${pic%.ncl}*"
   time ncl ${ncl}
   echo "--------------------------------------------------DONE ${ncl}"
done

cd $DIR_FIG

rm -f crop_crop_*
for pdf in `ls *pdf|grep -v crop`;do
    [[ ! -e crop_${pdf//crop_/} ]] && pdfcrop --margins 10 ${pdf} crop_${pdf} && echo "pdfcrop ${pdf} to crop_${pdf}"
    [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY -limit memory 64 -limit map 128 crop_${pdf} ${pdf%.pdf}.gif && echo "converting crop_${pdf} to ${pdf%.pdf}.gif"
#    [[ ! -e ${pdf%.pdf}.gif ]] && convert -density $DENSITY crop_${pdf} ${pdf%.pdf}.gif && echo "converting crop_${pdf} to ${pdf%.pdf}.gif"
#    convert -density $DENSITY -resize 220x170 crop_${pdf} thumbnail_${pdf%.pdf}.gif
done    

#for ps in `ls *ps`;do
#    convert -density $DENSITY -trim -bordercolor white -border 5x5 ${ps} ${ps%.ps}.gif
#done    

echo "===================================================================================================="
echo "DONE: compare3, MODEL COMPARISION, only the first run of each model are used for plotting."
echo "===================================================================================================="
cd -
;;

#====================================================================================================
#--- out of range  
*)
echo "WRONG compare !!!, should be 1, 2 or 3"
;;
esac

done ### exp loop

cd ../
find figs -depth -type f |grep -v 'ps$' > a.txt
tim=`date "+%Y%m%d%H%M%S"`
[[ ${tar_figs} -eq "1" ]] && tar -cvf figs_${tim}.tar `cat a.txt` > /dev/null 2>&1
