load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/shea_util.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl" 

begTime = get_cpu_time()

;--------------------------------------------------------------------------------------------------- User defined functions
;--------------------------------------------------------------------------------------------------- get_lat_lon(fin,varname)
;--- returen [strs] as the coordinates names of variable [varname] in file [fin]
undef ("get_lat_lon") ;-- optional
function get_lat_lon(fin)
local strs
begin
  strs=(/"lat", "lon"/)
  if(isfilevar(fin,"latitude"))then
     strs=(/"latitude", "longitude"/)
  end if
  if(isfilevar(fin,"nav_lat"))then
     strs=(/"nav_lat", "nav_lon"/)
  end if
  if(isfilevar(fin,"lat"))then
     strs=(/"lat", "lon"/)
  end if
  print("strs:"+strs)
  return(strs)
end

;------------------------------------------------------------------------------ getenv: directory, filenames, dimensions of Q T S, and period of this file
lat0=stringtodouble(getenv("LAT0"))
lon_west = stringtodouble(getenv("LON_WEST"))
lon_east = stringtodouble(getenv("LON_EAST"))
print("lat0: "+lat0+", lon_west: "+lon_west+", lon_east: "+lon_east)

Tmin = stringtodouble(getenv("Tmin"))
Tmax = stringtodouble(getenv("Tmax"))
Smin = stringtodouble(getenv("Smin"))
Smax = stringtodouble(getenv("Smax"))
   
run=getenv("RUN")
dT=stringtodouble(getenv("DT"))
dS=stringtodouble(getenv("DS"))
resol=getenv("resol")
print("dT: "+dT+", dS: "+dS+", resol: "+resol)

DIR_in0    = getenv("DIR_THETAO")
DIR_in1    = getenv("DIR_SO")
DIR_in2    = getenv("DIR_VMO")
DIR_out    = getenv("DIR_OUT")

Tname=getenv("thetao")
Sname=getenv("so")
Vname=getenv("vo")
Qname=getenv("vmo")
print("T: "+Tname)
print("S: "+Sname)
print("V: "+Vname)
print("Q: "+Qname)

ncs0  = getenv("THETAO")
ncs1  = getenv("SO")
ncs2  = getenv("VMO")
print("T: "+DIR_in0+ncs0)
print("S: "+DIR_in1+ncs1)
print("Q: "+DIR_in2+ncs2)

model=getenv("MODEL")
yymm = tochar(getenv("YYMM"))
print("model: "+model+", period: "+yymm)
ya=toint(tostring(yymm(0:3)))
ma=toint(tostring(yymm(4:5)))
yb=toint(tostring(yymm(7:10)))
mb=toint(tostring(yymm(11:12)))

;------------------------------------------------------------------------------ set t1 t2 s1 s2
za=Tmin+dT/2
zb=Tmax-dT/2
zmt=round((Tmax-Tmin)/dT,3)

sa=Smin+dS/2
sb=Smax-dS/2
smt=round((Smax-Smin)/dS,3)

tmin=-200
tmax=3200
smin=32000
smax=38000

print(za+" "+zb+" "+sa+" "+sb)
print(zmt+" "+smt)

thetas = fspan(za, zb, zmt)
salt   = fspan(sa, sb, smt)
t1     = round((thetas - dT/2)*100,3)
t2     = round((thetas + dT/2)*100,3)
s1     = round((salt   - dS/2)*1000,3)
s2     = round((salt   + dS/2)*1000,3)
eps = 1e-6
epst= 1e-3
epss= 1e-3

;------------------------------------------------------------------------------ loading T S Q & dims
setfileoption("nc","Format","LargeFile")

  nc  = DIR_in0+ncs0
  print("Loading...   "+nc)
  fin0 = addfile(nc,"r") 
  filevardimnamesT = getfilevardims(fin0,Tname)
  printVarSummary(filevardimnamesT)
  print(""+filevardimnamesT)
  latlonT=get_lat_lon(fin0)
  print(Tname+" Dimensions: "+filevardimnamesT(0)+" "+filevardimnamesT(1)+" "+latlonT(0)+" "+latlonT(1))

  nc  = DIR_in1+ncs1
  print("Loading...   "+nc)
  fin1 = addfile(nc,"r") 

  nc  = DIR_in2+ncs2
  print("Loading...   "+nc)
  fin2 = addfile(nc,"r") 
  if (isfilevar(fin2, Vname)) then
     vvv=Vname
     if(isfilevar(fin2,"lev_bnds"))then
        bnd="lev_bnds"
     else if (isfilevar(fin2,"depth_bnds"))then
        bnd="depth_bnds"
     else if (isfilevar(fin2,"olevel_bounds"))then
        bnd="olevel_bounds"
     else
        print("lev_bnds or depth_bnds NOT EXISTS, STOP!")
        exit
     end if
     end if
     end if
  else
     vvv=Qname
  end if
  
  filevardimnamesQ = getfilevardims(fin2,vvv)
  latlonQ=get_lat_lon(fin2)
  print("vmo    Dimensions: "+filevardimnamesQ(0)+" "+filevardimnamesQ(1)+" "+latlonQ(0)+" "+latlonQ(1))

  TIM= fin2->$filevardimnamesQ(0)$
  LEV= fin2->$filevardimnamesQ(1)$
  LAT  = fin2->$latlonQ(0)$
  LON  = fin2->$latlonQ(1)$
  dlon = where(LON.lt.360,LON+360,LON)
  dlon = mod(dlon,360) ; then 0<=dlon<360.
  LON  = (/dlon/)
  delete(dlon)

  kmt=dimsizes(LEV)

  tmt=dimsizes(TIM)
  if(tmt.lt.12)then
     print(model+" "+yymm+" file is short than 12 months, PLZ check!")
     exit
;     continue
  end if
;------------------------------------------------------------------------------ locate 30W
  Ndim_lon=dimsizes(dimsizes(LON))
  if(Ndim_lon.eq.2)then
     imt=dimsizes(LON(0,:))
     jmt=dimsizes(LON(:,0))
     xxx=LON(jmt/2,:)
     i30w=minind(abs(xxx-330))
     yyy=LAT(:,i30w)
     jlat0=minind(abs(yyy-lat0))
     lon1 = LON(jlat0,:)
     lat1 = LAT(jlat0,:)
  else
     imt=dimsizes(LON)
     jmt=dimsizes(LAT)
     xxx=LON
     i30w=minind(abs(xxx-330))
     yyy=LAT
     jlat0=minind(abs(yyy-lat0))
     lon1 = LON
     lat1 = lon1
     lat1 = LAT(jlat0)
  end if
;------------------------------------------------------------------------------ dx

  if (vvv.eq."vo")
     LEV_BNDS= fin2->$bnd$
     dz = abs(LEV_BNDS(:,1) - LEV_BNDS(:,0))
     lon2 = lon1
     lat2 = lat1
     lat2(0:imt-2) = lat1(1:imt-1)
     lon2(0:imt-2) = lon1(1:imt-1)
     lat2(imt-1)   = lat1(0)
     lon2(imt-1)   = lon1(0)
     dist = gc_latlon(lat1,lon1, lat2,lon2, 2,3)
     dx = dist
     dx(1:imt-1) =( dist(0:imt-2) + dist(1:imt-1) )/2.
     dx(0)       =( dist(imt-1)   + dist(0)       )/2.
     dx_conform = conform_dims((/tmt,kmt,imt/),tofloat(dx),(/2/))
     dz_conform = conform_dims((/tmt,kmt,imt/),tofloat(dz),(/1/))
  end if  

  print("30W: "+i30w+" "+xxx(i30w)+"E")

;------------------------------------------------------------------------------ locate lat0: 26.5 and its west/east boundaries
  ja=jlat0
  jb=jlat0
  print(lat0+"N: "+jlat0+" "+yyy(jlat0))

  iwest=minind(abs(xxx-lon_west))
  ieast=minind(abs(xxx-lon_east))
  print("iwest: "+iwest+" "+xxx(iwest)+"E")
  print("ieast: "+ieast+" "+xxx(ieast)+"E")
  if(iwest.lt.ieast)then
     idx=ispan(iwest,ieast,1)
  else
     idx0=ispan(1,ieast,1)
     idx1=ispan(iwest,imt-1,1)
     len0=dimsizes(idx0)
     len1=dimsizes(idx1)
     idx=new(len0+len1,integer,-9999)
     idx(0:len0-1)=idx0
     idx(len0:len0+len1-1)=idx1
  end if

;------------------------------------------------------------------------------ if staggered grids, perform interpolation
  flag=0
  if(num(ismissing(fin0->$Tname$(0,0,jlat0,:))) .ne. num(ismissing(fin2->$vvv$(0,0,jlat0,:))))then
     print("staggered grids: T/S differs from Q. Interpolate T/S onto Q grid.")
     if(Ndim_lon.eq.2)then
        LATT    = fin0->$latlonT(0)$(ja,:)
        LONT    = fin0->$latlonT(1)$(ja,:)
        LATQ    = fin2->$latlonQ(0)$(ja,:)
        LONQ    = fin2->$latlonQ(1)$(ja,:)
     else
        LATT    = fin0->$latlonT(0)$(ja)
        LONT    = fin0->$latlonT(1)$
        LATQ    = fin2->$latlonQ(0)$(ja)
        LONQ    = fin2->$latlonQ(1)$
     end if
     flag=1
  end if
;---------------------------------------------------------------------------------------------------- yearly Q(T,S)
ta=0
do year=ya,yb
   print(""+year)
   ta=(year-ya)*12
   tb=ta+11
   time= TIM(ta:tb)
   nc = DIR_out+"QTS_"+model+"_"+run+"_"+year+"_"+jlat0+"_"+lat0+".nc"
if(isfilepresent(nc))then         ;------------------- if QTS*.nc already exists, load Q|T|S, else get Q|T|S by extract or interpolat
   print("QTS exists.")
   fin3 = addfile(nc,"r")
   Ttmp = fin3->thetao
   Stmp = fin3->so
   Qtmp = fin3->vmo
else   
   ;------------------------------------------------------------------------------------------------- load Q T S and interpolate T/S if staggered grid
   if (flag.eq.0)then
      print("same grid: calculate vmo")
      Ttmp = fin0->$Tname$(ta:tb,:,jlat0,idx)
      Stmp = fin1->$Sname$(ta:tb,:,jlat0,idx)
      Qtmp = fin2->$vvv$(ta:tb,:,jlat0,idx)
      print("valid num of Q T V: "+num(.not.ismissing(Qtmp))+" "+num(.not.ismissing(Ttmp))+" "+num(.not.ismissing(Stmp)))
      Qtmp@_FillValue = 1e20
      Ttmp@_FillValue = 1e20
      Stmp@_FillValue = 1e20
      if(model.eq."E3SM-1-0")then
         Qtmp=where(Qtmp.gt.0.999999,Qtmp@_FillValue,Qtmp)
         Stmp=where(Stmp.le.1.000001,Stmp@_FillValue,Stmp)
         Ttmp=where(Stmp.le.1.000001,Ttmp@_FillValue,Ttmp)
      end if
      if(model.eq."INM-CM4-8")then
         Stmp=where(Stmp.le.1.000001,Stmp@_FillValue,Stmp)
         Ttmp=where(Stmp.le.1.000001,Ttmp@_FillValue,Ttmp)
      end if
      Ttmp=where(ismissing(Qtmp), Ttmp@_FillValue, Ttmp)
      Stmp=where(ismissing(Qtmp), Stmp@_FillValue, Stmp)
      if(num(.not.ismissing(Qtmp)) .ne. num(.not.ismissing(Ttmp)) .or. num(.not.ismissing(Ttmp)) .ne. num(.not.ismissing(Stmp)))then
         print("bad. valid num of Q T V: "+num(.not.ismissing(Qtmp))+" "+num(.not.ismissing(Ttmp))+" "+num(.not.ismissing(Stmp)))
         ;------------------------ fill land with nearby sea value
         varTi = linmsg(Ttmp,-1)               ;; linmsg(x,0)   ; endPoint= 0 missing end point(s) unaltered
                                               ;; linmsg(y,-1)  ; endPoint= -1 missing end point(s) set to nearest non-missing value
         print(""+num(.not.ismissing(varTi)))
         varTi = linmsg_n(varTi,-1,1)
         varSi = linmsg(Stmp,-1)
         varSi = linmsg_n(varSi,-1,1)
         copy_VarMeta(Ttmp,varTi)
         copy_VarMeta(Stmp,varSi)

         copy_VarCoords(Qtmp,varTi)
         copy_VarCoords(Qtmp,varSi)
         Ttmp = varTi  ;;;(:,:,idx)
         Stmp = varSi  ;;;(:,:,idx)
         Ttmp = where(ismissing(Qtmp),Qtmp@_FillValue,Ttmp)
         Stmp = where(ismissing(Qtmp),Qtmp@_FillValue,Stmp)
         delete(varTi)
         delete(varSi)
      end if
      if(vvv.eq."vo")then
         Qtmp = (/Qtmp * dx_conform(ta:tb,:,idx) * dz_conform(ta:tb,:,idx)/)
      end if
   else
      print("staggered grid: interpolate")
      print(ta+" "+tb+" "+ja+" "+sizeof(idx)+" "+kmt)
      varT = fin0->$Tname$(ta:tb,:,ja,idx)
      print(""+sizeof(varT))
      varS = fin1->$Sname$(ta:tb,:,ja,idx)
      varQ = fin2->$vvv$(ta:tb,:,ja,idx)
      varT@_FillValue = 1e20
      varS@_FillValue = 1e20
      if(model.eq."E3SM-1-0")then
         varQ=where(varQ.gt.0.999999,varQ@_FillValue,varQ)
         varS=where(varS.lt.1.000001,varS@_FillValue,varS)
         varT=where(varS.lt.1.000001,varT@_FillValue,varT)
      end if
      if(model.eq."INM-CM4-8")then
         varS=where(varS.le.1.000001,varS@_FillValue,varS)
         varT=where(varS.le.1.000001,varT@_FillValue,varT)
      end if
      varT=where(ismissing(varQ), varT@_FillValue, varT)
      varS=where(ismissing(varQ), varS@_FillValue, varS)
      print(""+num(.not.ismissing(varT)))
      if(vvv.eq."vo")then
         varQ = (/varQ * dx_conform(ta:tb,:,idx) * dz_conform(ta:tb,:,idx)/)
         print(""+num(.not.ismissing(varQ)))
      end if
      ;------------------------ fill land with nearby sea value
      varTi = linmsg(varT,-1)               ;; linmsg(x,0)   ; endPoint= 0 missing end point(s) unaltered
                                            ;; linmsg(y,-1)  ; endPoint= -1 missing end point(s) set to nearest non-missing value
      print(""+num(.not.ismissing(varTi)))
      varTi = linmsg_n(varTi,-1,1)
      varSi = linmsg(varS,-1)
      varSi = linmsg_n(varSi,-1,1)
      copy_VarMeta(varT,varTi)
      copy_VarMeta(varS,varSi)

      copy_VarCoords(varQ,varTi)
      copy_VarCoords(varQ,varSi)
      Ttmp = varTi  ;(:,:,idx)
      Stmp = varSi  ;(:,:,idx)
      Qtmp =  varQ  ;(:,:,idx)
      Ttmp = where(ismissing(Qtmp),Qtmp@_FillValue,Ttmp)
      Stmp = where(ismissing(Qtmp),Qtmp@_FillValue,Stmp)
      delete(varT)
      delete(varS)
      delete(varQ)
      delete(varTi)
      delete(varSi)
   end if

   Qtmp=where(Qtmp.gt.1e20,1e20,Qtmp)
   if(max(abs(Qtmp(0,:,:))).gt.1e7)then
      Qtmp= Qtmp/1e9
   end if
   if(max(abs(Qtmp(0,:,:))).gt.1e4)then
      Qtmp= Qtmp/1e6
   end if

   
   tmp=(/Qtmp/)
   copy_VarAtts(Qtmp, tmp)
   copy_VarCoords_1(Qtmp,tmp)
   tmp!1="lev"
   tmp!2="lon"
   lon0=lon1(idx)
   lon0=where(lon0.gt.180,lon0-360,lon0)
   ip=dim_pqsort(lon0, 2) 
   tmp&lon=lon0
   printMinMax(lon0,False)

   print("Saving: "+nc)
   system("rm "+nc)
   fout = addfile(nc,"c")

   tmp=(/Qtmp(:,:,ip)/)
   copy_VarAtts(Qtmp, tmp)
   fout->vmo    = tmp

   tmp=(/Ttmp(:,:,ip)/)
   copy_VarAtts(Ttmp, tmp)
   fout->thetao = tmp

   tmp=(/Stmp(:,:,ip)/)
   copy_VarAtts(Stmp, tmp)
   fout->so     = tmp

end if

;exit

   nc = DIR_out+"trans_"+resol+"_"+model+"_"+run+"_"+year+"_"+jlat0+"_"+lat0+".nc"
if(isfilepresent(nc))then         ;------------------- if trans*.nc already exists, cycle, else calculate it here
   continue
end if
   
   if(num(.not.ismissing(Qtmp)) .ne. num(.not.ismissing(Ttmp)) .or. num(.not.ismissing(Ttmp)) .ne. num(.not.ismissing(Stmp)))then
      print("bad. valid num of Q T V: "+num(.not.ismissing(Qtmp))+" "+num(.not.ismissing(Ttmp))+" "+num(.not.ismissing(Stmp)))
;      if(model.eq."GFDL-CM4" .or. model.eq."INM-CM4-8" .or. model.eq."INM-CM5-0")then
;         print(""+model)
;      else
         exit
;      end if
   end if   
   print("Q T S sea number: "+num(.not.ismissing(Qtmp))+" "+num(.not.ismissing(Ttmp))+" "+num(.not.ismissing(Stmp)))

   ;------------------------------------------------------------------------------------------------- convert Q T S's units to Sv, DegC and psu respectively
   if(max(Ttmp(0,:,:)).gt.100)then
      Ttmp= Ttmp - 273.15
   end if
   thetao = todouble(Ttmp) * 100
   copy_VarMeta(Ttmp,thetao)
   delete(Ttmp)

   if(max(Stmp(0,:,:)).lt.10)then
      Stmp= Stmp * 1000.
   end if
   so = todouble(Stmp) * 1000
   copy_VarMeta(Stmp,so)
   delete(Stmp)

   vmo = todouble(Qtmp)
   copy_VarMeta(Qtmp,vmo)
   vmo=where(vmo.gt.1e19,1e20,vmo)
   delete(Qtmp)
   if(max(abs(vmo(0,:,:))).gt.1e7)then
      vmo= vmo/1e9
   end if
   if(max(abs(vmo(0,:,:))).gt.1e4)then
      vmo= vmo/1e6
   end if

   midTime=get_cpu_time()
   print("loading &| interpolation time:" + (midTime - begTime))
   ;---------------------------------------------------------------------------------------------------- yearly Q(T,S)
   print("==================== "+model+": "+year+" of "+yymm+" Data Loading OK ====================")
   tmv = dimsizes(vmo(:,0,0))
   kmv = dimsizes(vmo(0,:,0))
   imv = dimsizes(vmo(0,0,:))

   trans = new((/1,zmt,smt,1/),typeof(vmo), 1e20)
   trans = 0

   ii0=0
   jj0=0
   
   if(all(ismissing(thetao(0,:,:))))then
      continue
   end if

   qn=ndtooned(vmo(:,:,:))
   tn=ndtooned(thetao(:,:,:))
   sn=ndtooned(so(:,:,:))
   
   ;---------------------------------------------------------------------------------------------------- narrow T S to (tmin, tmax), (smin,smax) respectively
   if(min(tn).le.tmin)then
      tn( ind(tn.le.tmin) ) = (tmin/100.+epst)*100.
   end if
   if(max(tn).ge.tmax)then
      tn( ind(tn.ge.tmax) ) = (tmax/100.-epst)*100.
   end if
   if(min(sn).le.smin)then
      sn( ind(sn.le.smin) ) = (smin/1000.+epss)*1000.
   end if
   if(max(sn).ge.smax)then
      sn( ind(sn.ge.smax) ) = (smax/1000.-epss)*1000.
   end if

   ;------------------------------------------ T @ lat0 are only within part of  t range, traverse within this temperature range
   jl=ind( min(tn).lt.t2 .and. max(tn).ge.t1 )
   if(all(ismissing(jl)))then
      delete(jl)
      continue
   end if
   njl=dimsizes(jl)
   
   do j0=0,njl-1
      z=jl(j0)   ;;; z: index for zeta
      ii = ind( tn.lt.t2(z) .and. tn.ge.t1(z) )
      if(all(ismissing(ii)))then
         delete(ii)
         continue
      end if
      nii=dimsizes(ii)
      ii0=ii0+nii
      qs=qn(ii) 
      ss=sn(ii) 
      ;------------------------------------------ there are a bunch of grids are within [t1 t2], their salinity's range are small, thus traverse only within this salinity range
      il=ind( min(ss).lt.s2 .and. max(ss).ge.s1 )
      if(all(ismissing(il)))then
         delete(il)
         delete(ii)
         delete(qs)
         delete(ss)
         continue
      end if
      nil=dimsizes(il)
      kk0=0
      do i0=0,nil-1
         s=il(i0)  ;;; s: index for salt
         ;------------------------------------------ there are a few of grids are within [t1 t2] & [s1, s2], sum their Q
         ii1 = ind( ss.lt.s2(s) .and. ss.ge.s1(s) )
         if(all(ismissing(ii1)))then
            delete(ii1)
            continue
         end if
         nii1=dimsizes(ii1)
         jj0=jj0+nii1
         kk0=kk0+nii1
         trans(0,z,s,0)=sum(qs(ii1))
         ;--------------------- Once used, kick out from the traverse
         qs(ii1)=0
         ss(ii1)=ss@_FillValue
         delete(ii1)
      end do ;;; salt loop
      sub=nii-kk0
   if(sub.ne.0)then
     print("inexact: not all salinity counted")
     print("nii:"+nii+" kk0:"+kk0+" sub:"+sub)
     print("qs:"+qs+" ss:"+ss+" thetas:"+thetas(z))
     print("min(ss):"+min(ss)+" max(ss):"+max(ss))
     print("sum(qs)/eps:"+sum(qs)/eps)
;     if(model.eq."GFDL-CM4" .or. model.eq."INM-CM4-8" .or. model.eq."INM-CM5-0")then
;        print(""+model)
;     else
        exit
;     end if
   end if
      qn(ii)=0
      sn(ii)=sn@_FillValue
      tn(ii)=tn@_FillValue
      delete(ii)
      delete(qs)
      delete(ss)
      delete(il)
   end do ;;; theta loop
   delete(jl)
   delete(qn)
   delete(tn)
   delete(sn)
   dum = abs(sum(vmo(:,:,:))-sum(trans(:,:,:,0)))
   if(dum .gt. eps)then
;      print("inexact: "+ dum/eps+" "+j+" "+lat(j))
      print("inexact: "+ dum/eps)
;      if(model.eq."GFDL-CM4" .or. model.eq."INM-CM4-8" .or. model.eq."INM-CM5-0")then
;         print(""+model)
;      else
         exit
;      end if
   end if
   print(ii0+" "+jj0)
      
   qtsTime = get_cpu_time()
   print("Q(T,S) time:" + (qtsTime - midTime))
   ;---------------------------------------------------------------------------------------------------- assign attributes
   trans = trans/(tb-ta+1)
   thetas@long_name = "theta"
     salt@long_name = "salt"
   thetas@units     = "DegC"
     salt@units     = "psu"
   trans=where(trans.eq.0,trans@_FillValue,trans)
   trans!0="time"
   trans!1="theta"
   trans!2="salt"
   trans!3="lat"
   trans&time  = dim_avg_Wrap(time)
   trans&theta = thetas
   trans&salt  = salt
   trans&lat   = lat0
   trans@long_name="V(T,S)"
   trans@units    ="Sv"
   ;---------------------------------------------------------------------------------------------------- save this year's Q(T,S)
   nc = DIR_out+"trans_"+resol+"_"+model+"_"+run+"_"+year+"_"+jlat0+"_"+lat0+".nc"
   print("Saving: "+nc)
   system("rm "+nc)
   fout = addfile(nc,"c")
   ;=============================================================================
   fout->trans = trans
;   printVarSummary(trans)
;   printMinMax(trans,False)
   delete(trans)
end do ;;; year loop

print(model+": "+yymm)
print("TOTAL TIME: " + (get_cpu_time() - begTime))

